
import java.util.*;
import java.io.*;

public class levenshtein_font
{
   public static void main(String[] args) throws Exception
   {
      PrintWriter out = new PrintWriter(System.out);
      new levenshtein_font(new FastScanner(System.in), out);
      out.close();
   }

   Node root = new Node();

   int get_c(char c)
   {
      return c == '?' ? 26 : c-'A';
   }

   void add(Node cur, int k, String s)
   {
      cur.numOps = Math.min(cur.numOps, k);
      for (int i=0; i<s.length(); i++)
      {
         int c = get_c(s.charAt(i));
         cur = cur.step(c);
      }
      cur.numOps = Math.min(cur.numOps, k);
      cur.isTerm = true;
   }

   long MODO = 998_244_353;
   char[] input;
   int K;

   // Find all patterns with k or less edits
   void gen(int i, int k, String s)
   {
      if (k < 0)
         return;

      if (i < input.length)
      {
         // Consider taking a real letter
         gen(i+1, k, s+input[i]);

         // Consider deleting a letter
         gen(i+1, k-1, s);

         // Consider replacing a letter
         gen(i+1, k-1, s+'?');
      }
      else
      {
         // Add to the possible patterns
         add(root, K-k, s); 
      }

      // Consider adding a dummy
      gen(i, k-1, s+'?');
   }

   HashMap<ArrayList<Integer>, Long> memo = new HashMap<>();
   long go(State cur)
   {
      ArrayList<Integer> trace = cur.getTrace();
      Long rres = memo.get(trace);
      if (rres != null)
         return rres;

      long res = cur.count(K);
      for (int c=0; c<26; c++)
      {
         State nxt = cur.step(c);
         if (nxt.poses.size() > 0)
         {
            res += go(nxt);
            res %= MODO;
         }
      }

      memo.put(trace, res);
      return res;
   }

   public levenshtein_font(FastScanner in, PrintWriter out)
   {
      input = in.next().toCharArray();
      K = in.nextInt();

      gen(0, K, "");

      State cur = new State();
      cur.poses.add(root);
      root.trim(K);

      long res = go(cur);
      out.println(res);
   }
}

class State
{
   ArrayDeque<Node> poses;
   State()
   {
      poses = new ArrayDeque<>();
   }

   int count(int K)
   {
      int res = 0;
      for (Node cur : poses)
         if (cur.isTerm)
         {
            if (cur.numOps < K)
               return 0;
            res = 1;
         }
   
      return res;
   }

   ArrayList<Integer> getTrace()
   {
      int ptr = 0;
      ArrayList<Integer> trace = new ArrayList<>();
      for (Node n : poses)
         trace.add(n.id);
      Collections.sort(trace);
      return trace;
   }

   State step(int c)
   {
      State nxt = new State();
      for (Node cur : poses)
      {
         if (cur.nxt[c] != null)
            nxt.poses.add(cur.nxt[c]);
         if (cur.nxt[26] != null)
            nxt.poses.add(cur.nxt[26]);
      }
      return nxt;
   }
}

class Node
{
   static int counter = 0;
   boolean isTerm;
   int numOps, id;
   Node[] nxt;

   Node()
   {
      id = counter++;
      isTerm = false;
      numOps = 100;
      nxt = new Node[27];
   }

   Node step(int c)
   {
      if (nxt[c] == null)
         nxt[c] = new Node();
      return nxt[c];
   }

   boolean trim(int k)
   {
      boolean hit = isTerm && numOps == k;
      for (int c=0; c<27; c++)
      {
         if (nxt[c] != null)
         {
            boolean hasGood = nxt[c].trim(k);
            if (hasGood)
               hit = true;
            else
            {
               nxt[c] = null;
               System.out.println("CUT");
            }
         }
      }
      return hit;
   }
}

class FastScanner
{
   private InputStream stream;
   private byte[] buf = new byte[1024];
   private int curChar;
   private int numChars;
    
   public FastScanner(InputStream stream)
   {
      this.stream = stream;
   }
    
   int read()
   {
      if (numChars == -1)
         throw new InputMismatchException();
      if (curChar >= numChars)
      {
         curChar = 0;
         try
         {
            numChars = stream.read(buf);
         } 
         catch (IOException e) 
         {
            throw new InputMismatchException();
         }
         
         if (numChars <= 0)
            return -1;
      }
      return buf[curChar++];
   }
    
   boolean isSpaceChar(int c)
   {
      return c==' '||c=='\n'||c=='\r'||c=='\t'||c==-1;
   }
   
   boolean isEndline(int c)
   {
      return c=='\n'||c=='\r'||c==-1;
   }
   
   int nextInt()
   {
      return Integer.parseInt(next());
   }
   
   long nextLong()
   {
      return Long.parseLong(next());
   }
   
   double nextDouble()
   {
      return Double.parseDouble(next());
   }
   
   String next()
   {
      int c = read();
      while (isSpaceChar(c))
         c = read();
      StringBuilder res = new StringBuilder();
      do
      {
         res.appendCodePoint(c);
         c = read();
      } while(!isSpaceChar(c));
      return res.toString();
   }
   
   String nextLine()
   {
      int c = read();
      while (isEndline(c))
         c = read();
      
      StringBuilder res = new StringBuilder();
      do
      {
         res.appendCodePoint(c);
         c = read();
      } while(!isEndline(c));
      return res.toString();
   }
}
